@@ -5,15 +5,15 @@ use anyhow::Context;
 use rr_util::{
     name::Name,
     symbolic_size::{SymbolicSizeConstraint, SymbolicSizeConstraints},
-    tensor_util::{Shape, TorchDeviceDtypeOp},
+    tensor_util::TorchDeviceDtypeOp,
     util::HashBytes,
     IndexSet,
 };
 use uuid::uuid;
 
 use crate::{
-    CachedCircuitInfo, Circuit, CircuitFlags, CircuitNode, CircuitRc, ConstructError, NamedAxes,
-    Result, Symbol,
+    CachedCircuitInfo, CircuitFlags, CircuitNode, CircuitRc, ConstructError, NamedAxes, Result,
+    Symbol,
 };
 
 pub trait CircuitNodeInit {
