@@ -31,6 +31,7 @@ use rr_util::{
     },
 };
 use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
+use smallvec::ToSmallVec;
 use uuid::uuid;
 
 use crate::{
@@ -53,32 +54,29 @@ use crate::{
 #[pyclass(extends=PyCircuitBase)]
 #[derive(Clone)]
 pub struct Einsum {
-    pub args: EinsumArgs,
+    pub in_axes: Vec<EinsumAxes>,
     pub out_axes: EinsumAxes,
     info: CachedCircuitInfo,
 }
 
 circuit_node_extra_impl!(Einsum);
 
-impl CircuitNodeComputeInfoImpl for Einsum {
-    fn compute_shape(&self) -> Shape {
-        self.info().shape.clone()
-    }
-}
+impl CircuitNodeComputeInfoImpl for Einsum {}
 impl CircuitNodeHashWithChildren for Einsum {
     fn compute_hash_non_name(&self, hasher: &mut blake3::Hasher) {
         hasher.update(&self.out_axes);
-        for (c, axes) in &self.args {
+        for (c, axes) in self.args() {
             // no need to delimit because we have hash each loop
             hasher.update(&c.info().hash);
-            hasher.update(axes);
+            hasher.update(&axes);
         }
+        ()
     }
 }
 impl CircuitNodeSelfOnlyHash for Einsum {
     fn compute_self_only_hash(&self, hasher: &mut blake3::Hasher) {
         hasher.update(&self.out_axes);
-        for (_c, axes) in &self.args {
+        for axes in &self.in_axes {
             hasher.update(uuid!("3749ab7f-4f4c-4fdb-a268-af9c084e70ef").as_bytes());
             hasher.update(axes);
         }
@@ -92,7 +90,8 @@ impl CircuitNode for Einsum {
         let out_inv: HashMap<AxisInt, usize> = hashmap_collect_except_duplicates(
             self.out_axes.iter().enumerate().map(|(i, x)| (*x, i)),
         );
-        self.input_axes()
+        self.in_axes
+            .iter()
             .map(|ints| {
                 ints.iter()
                     .map(|i| {
@@ -106,22 +105,13 @@ impl CircuitNode for Einsum {
             .collect()
     }
 
-    fn children(&self) -> Box<dyn Iterator<Item = CircuitRc> + '_> {
-        Box::new(self.args.iter().map(|a| a.0.clone()))
-    }
-
-    fn map_children_enumerate<F>(&self, mut f: F) -> Result<Self>
-    where
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>,
-    {
+    fn _replace_children(&self, children: Vec<CircuitRc>) -> Result<Self> {
         Self::try_new(
-            self.args
+            children
                 .iter()
-                .enumerate()
-                .map(move |(i, (circ, axes))| -> Result<_> {
-                    Ok((f(i, circ.clone())?, axes.clone()))
-                })
-                .collect::<Result<_, _>>()?,
+                .zip(&self.in_axes)
+                .map(|(c, axes)| (c.clone(), axes.clone()))
+                .collect(),
             self.out_axes.clone(),
             self.info().name,
         )
@@ -134,13 +124,13 @@ impl CircuitNode for Einsum {
     fn eval_tensors(&self, tensors: &[Tensor]) -> Result<Tensor> {
         let tensors = upcast_tensors_for_eval!(self, tensors);
         Python::with_gil(|py| {
-            if self.args.is_empty() {
+            if self.in_axes.is_empty() {
                 return Ok(scalar_to_tensor_py(py, 1., sv![], Default::default())?);
             }
             let out_axes_deduped: EinsumAxes = self.out_axes.iter().unique().cloned().collect();
             let result_non_diag: Tensor = einsum_py(
                 py,
-                zip(tensors.iter().cloned(), self.input_axes().cloned()).collect(),
+                zip(tensors.iter().cloned(), self.in_axes.iter().cloned()).collect(),
                 out_axes_deduped.clone(),
             )?;
 
@@ -160,22 +150,18 @@ impl CircuitNode for Einsum {
 
 pub type EinsumArgs = Vec<(CircuitRc, EinsumAxes)>;
 impl Einsum {
-    pub fn input_circuits(&self) -> Box<dyn Iterator<Item = CircuitRc> + '_> {
-        Box::new(self.args.iter().map(|(circ, _)| circ.clone()))
-    }
-
-    pub fn input_axes(&self) -> Box<dyn Iterator<Item = &EinsumAxes> + '_> {
-        Box::new(self.args.iter().map(|(_, axes)| axes))
-    }
-
-    pub fn shape_map(&self) -> Result<HashMap<AxisInt, usize>> {
+    fn shape_map_(
+        children: &Vec<CircuitRc>,
+        in_axes: &Vec<EinsumAxes>,
+        name: Option<Name>,
+    ) -> Result<HashMap<AxisInt, usize>> {
         let mut out = HashMap::default();
-        for (circ, axes) in &self.args {
+        for (circ, axes) in zip(children, in_axes) {
             if circ.info().rank() != axes.len() {
                 bail!(ConstructError::EinsumLenShapeDifferentFromAxes {
                     len_axes: axes.len(),
                     child_len_shape: circ.info().rank(),
-                    einsum_name: self.info().name,
+                    einsum_name: name,
                     child_circuit: circ.clone(),
                 });
             }
@@ -186,7 +172,7 @@ impl Einsum {
                         new_size: circuit_size.into(),
                         existing_size: existing_size.into(),
                         axis: *axis as usize,
-                        einsum_name: self.info().name,
+                        einsum_name: name,
                         child_circuit: circ.clone(),
                     });
                 }
@@ -196,30 +182,34 @@ impl Einsum {
         Ok(out)
     }
 
+    pub fn shape_map(&self) -> Result<HashMap<AxisInt, usize>> {
+        Self::shape_map_(&self.info.children, &self.in_axes, self.info().name)
+    }
+
     #[apply(new_rc_unwrap)]
     pub fn try_new(args: EinsumArgs, out_axes: EinsumAxes, name: Option<Name>) -> Result<Self> {
-        // TODO: reduce axes nums!
-        let mut out = Self {
-            args,
-            out_axes,
-            info: Default::default(),
-        };
-        out.info.name = name;
-        let map = out.shape_map()?; // catch errors
+        let (children, in_axes) = args.into_iter().unzip();
+        let map = Self::shape_map_(&children, &in_axes, name)?;
 
-        out.info.shape = out
-            .out_axes
+        let shape = out_axes
             .iter()
             .map(|a| map.get(a).cloned())
             .collect::<Option<Shape>>()
             .ok_or_else(|| {
                 anyhow::Error::from(ConstructError::EinsumOutputNotSubset {
-                    circuit_name: out.info().name,
+                    circuit_name: name,
                     all_input_axes: map.into_keys().collect(),
-                    output_axes: out.out_axes.clone(),
+                    output_axes: out_axes.clone(),
                 })
             })?;
 
+        // TODO: reduce axes nums!
+        let out = Self {
+            out_axes,
+            info: CachedCircuitInfo::incomplete(name, shape, children),
+            in_axes,
+        };
+
         out.initial_init_info()
     }
 
@@ -245,7 +235,15 @@ impl Einsum {
 
     pub fn axes_in_input(&self) -> HashSet<AxisInt> {
         // could be 256 bit bitmask
-        self.input_axes().flatten().copied().collect()
+        self.in_axes.iter().flatten().copied().collect()
+    }
+
+    pub fn args(&self) -> impl Iterator<Item = (CircuitRc, &EinsumAxes)> {
+        self.children().zip(self.in_axes.iter())
+    }
+
+    pub fn args_cloned(&self) -> Vec<(CircuitRc, EinsumAxes)> {
+        self.args().map(|(a, b)| (a, b.clone())).collect()
     }
 }
 
@@ -295,9 +293,9 @@ impl Einsum {
         Ok(out.into_init())
     }
 
-    #[getter]
-    fn args(&self) -> PyEinsumArgs {
-        PyEinsumArgs(self.args.clone())
+    #[getter(args)]
+    fn args_py(&self) -> PyEinsumArgs {
+        PyEinsumArgs(self.args_cloned())
     }
 
     #[getter]
@@ -305,12 +303,12 @@ impl Einsum {
         PyEinsumAxes(self.out_axes.clone())
     }
 
-    fn all_input_circuits(&self) -> Vec<CircuitRc> {
-        self.input_circuits().collect()
+    fn all_input_axes(&self) -> Vec<PyEinsumAxes> {
+        self.in_axes.iter().cloned().map(PyEinsumAxes).collect()
     }
 
-    fn all_input_axes(&self) -> Vec<PyEinsumAxes> {
-        self.input_axes().cloned().map(PyEinsumAxes).collect()
+    fn all_input_circuits(&self) -> Vec<CircuitRc> {
+        self.info().children.clone()
     }
 
     #[staticmethod]
@@ -484,7 +482,7 @@ impl Einsum {
 
     pub fn evolve(&self, args: Option<EinsumArgs>, out_axes: Option<EinsumAxes>) -> Einsum {
         Einsum::try_new(
-            args.unwrap_or_else(|| self.args.clone()),
+            args.unwrap_or_else(|| self.args_cloned()),
             out_axes.unwrap_or_else(|| self.out_axes.clone()),
             self.info().name,
         )
@@ -492,7 +490,8 @@ impl Einsum {
     }
 
     pub fn reduced_axes(&self) -> U8Set {
-        self.input_axes()
+        self.in_axes
+            .iter()
             .flatten()
             .filter(|i| !self.out_axes.contains(i))
             .copied()
@@ -500,7 +499,8 @@ impl Einsum {
     }
 
     pub fn next_axis(&self) -> u8 {
-        self.input_axes()
+        self.in_axes
+            .iter()
             .flatten()
             .max()
             .map(|x| x + 1)
@@ -510,7 +510,7 @@ impl Einsum {
     pub fn get_spec(&self) -> EinsumSpec {
         let to_usize = |vals: &[u8]| -> Vec<usize> { vals.iter().map(|&x| x as usize).collect() };
         EinsumSpec {
-            input_ints: self.input_axes().map(|x| to_usize(x)).collect(),
+            input_ints: self.in_axes.iter().map(|x| to_usize(x)).collect(),
             output_ints: to_usize(&self.out_axes),
             int_sizes: dict_to_list(
                 &self
@@ -527,12 +527,7 @@ impl Einsum {
     pub fn normalize_ints(&self) -> Einsum {
         let spec = self.get_spec();
         let spec_normalized = spec.normalize();
-        Einsum::try_from_spec(
-            &spec_normalized,
-            &self.input_circuits().collect::<Vec<CircuitRc>>(),
-            self.info().name,
-        )
-        .unwrap()
+        Einsum::try_from_spec(&spec_normalized, &self.info.children, self.info().name).unwrap()
     }
 }
 
@@ -540,22 +535,23 @@ impl Einsum {
 #[pyclass(extends=PyCircuitBase)]
 #[derive(Clone)]
 pub struct Add {
-    #[pyo3(get)]
-    pub nodes: Vec<CircuitRc>,
     info: CachedCircuitInfo,
 }
 
 impl Add {
     #[apply(new_rc_unwrap)]
     pub fn try_new(nodes: Vec<CircuitRc>, name: Option<Name>) -> Result<Self> {
+        let shape = broadcast_shapes(
+            &nodes
+                .iter()
+                .map(|x| &x.info().shape[..])
+                .collect::<Vec<_>>(),
+        )
+        .with_context(|| format!("Sum isn't broadcastable name={:?}", name))?;
         // TODO: reduce axes nums!
-        let mut out = Self {
-            nodes,
-            info: Default::default(),
+        let out = Self {
+            info: CachedCircuitInfo::incomplete(name, shape, nodes),
         };
-        out.info.name = name;
-        out.compute_shape_maybe()
-            .with_context(|| format!("Sum isn't broadcastable name={:?}", out.info().name))?;
         out.initial_init_info()
     }
 
@@ -568,16 +564,6 @@ impl Add {
             name,
         )
     }
-
-    fn compute_shape_maybe(&self) -> Result<Shape> {
-        broadcast_shapes(
-            &self
-                .nodes
-                .iter()
-                .map(|x| &x.info().shape[..])
-                .collect::<Vec<_>>(),
-        )
-    }
 }
 
 #[pymethods]
@@ -592,14 +578,12 @@ impl Add {
 
     pub fn has_broadcast(&self) -> bool {
         !self
-            .nodes
-            .iter()
+            .children()
             .all(|node| node.info().shape == self.info().shape)
     }
 
     pub fn nodes_and_rank_differences(&self) -> Vec<(CircuitRc, usize)> {
-        self.nodes
-            .iter()
+        self.children()
             .map(|node| (node.clone(), self.info().rank() - node.info().rank()))
             .collect()
     }
@@ -607,8 +591,8 @@ impl Add {
     pub fn to_counts(&self) -> HashMap<CircuitRc, usize> {
         let mut counts = HashMap::default();
         counts.reserve(20);
-        for item in &self.nodes {
-            *counts.entry(item.clone()).or_insert(0) += 1;
+        for item in self.children() {
+            *counts.entry(item).or_insert(0) += 1;
         }
 
         counts
@@ -638,15 +622,16 @@ impl Add {
     pub fn minus(positive: CircuitRc, negative: CircuitRc, name: Option<Name>) -> Result<Self> {
         positive.sub(negative, name)
     }
+
+    #[getter]
+    pub fn nodes(&self) -> Vec<CircuitRc> {
+        self.info().children.clone()
+    }
 }
 
 circuit_node_extra_impl!(Add, self_hash_default);
 
-impl CircuitNodeComputeInfoImpl for Add {
-    fn compute_shape(&self) -> Shape {
-        self.compute_shape_maybe().unwrap() // assuming error was caught on creation, panicking now
-    }
-}
+impl CircuitNodeComputeInfoImpl for Add {}
 
 impl CircuitNodeHashItems for Add {}
 
@@ -673,26 +658,12 @@ impl CircuitNode for Add {
             .collect()
     }
 
-    fn children(&self) -> Box<dyn Iterator<Item = CircuitRc> + '_> {
-        Box::new(self.nodes.iter().cloned())
-    }
-
-    fn map_children_enumerate<F>(&self, mut f: F) -> Result<Self>
-    where
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>,
-    {
-        Self::try_new(
-            self.nodes
-                .iter()
-                .enumerate()
-                .map(move |(i, circ)| f(i, circ.clone()))
-                .collect::<Result<Vec<_>, _>>()?,
-            self.info().name,
-        )
+    fn _replace_children(&self, children: Vec<CircuitRc>) -> Result<Self> {
+        Self::try_new(children, self.info().name)
     }
 
     fn self_flops(&self) -> BigUint {
-        self.info.numel() * self.nodes.len()
+        self.info.numel() * self.num_children()
     }
 
     fn eval_tensors(&self, tensors: &[Tensor]) -> Result<Tensor> {
@@ -728,8 +699,6 @@ impl CircuitNodeAutoName for Add {
 #[pyclass(extends=PyCircuitBase)]
 #[derive(Clone)]
 pub struct Rearrange {
-    #[pyo3(get)]
-    pub node: CircuitRc,
     #[pyo3(get)]
     pub spec: Box<RearrangeSpec>,
     info: CachedCircuitInfo,
@@ -738,33 +707,33 @@ pub struct Rearrange {
 impl Rearrange {
     #[apply(new_rc_unwrap)]
     pub fn try_new(node: CircuitRc, spec: RearrangeSpec, name: Option<Name>) -> Result<Self> {
-        // spec must be valid because we enforce that on construction!
-        // TODO: reduce axes nums!
-        let mut out = Self {
-            node: node.clone(),
-            spec: Box::new(spec),
-            info: Default::default(),
-        };
-        out.info.name = name;
-
-        // Check that RearrangeSpec is valid for input, prevent panic on rearrange.info().shape
-        out.spec
+        // Check that RearrangeSpec is valid for input
+        let shape = spec
             .conform_to_input_shape(&node.info().shape)
             .with_context(|| {
                 format!(
                     "{}\nname={:?}, child={:?}",
                     "rearrange spec conform_to_input_shape failed in construct rearrange",
-                    out.info().name,
+                    name,
                     node
                 )
-            })?;
+            })?
+            .shapes()?
+            .1;
+
+        // spec must be valid because we enforce that on construction!
+        // TODO: reduce axes nums!
+        let out = Self {
+            spec: Box::new(spec),
+            info: CachedCircuitInfo::incomplete(name, shape, vec![node]),
+        };
 
         out.initial_init_info()
     }
 
     pub fn evolve(&self, node: Option<CircuitRc>, spec: Option<RearrangeSpec>) -> Rearrange {
         Rearrange::try_new(
-            node.unwrap_or_else(|| self.node.clone()),
+            node.unwrap_or_else(|| self.node().clone()),
             spec.unwrap_or_else(|| (*self.spec).clone()),
             self.info().name,
         )
@@ -793,11 +762,7 @@ impl Rearrange {
 
 circuit_node_extra_impl!(Rearrange, self_hash_default);
 
-impl CircuitNodeComputeInfoImpl for Rearrange {
-    fn compute_shape(&self) -> Shape {
-        self.conform_to_input_shape_spec().shapes().unwrap().1
-    }
-}
+impl CircuitNodeComputeInfoImpl for Rearrange {}
 
 impl CircuitNodeHashItems for Rearrange {
     fn compute_hash_non_name_non_children(&self, hasher: &mut blake3::Hasher) {
@@ -822,19 +787,8 @@ impl CircuitNode for Rearrange {
             .collect()]
     }
 
-    fn children(&self) -> Box<dyn Iterator<Item = CircuitRc> + '_> {
-        Box::new(std::iter::once(self.node.clone()))
-    }
-
-    fn map_children_enumerate<F>(&self, mut f: F) -> Result<Self>
-    where
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>,
-    {
-        Self::try_new(
-            f(0, self.node.clone())?,
-            (*self.spec).clone(),
-            self.info().name,
-        )
+    fn _replace_children(&self, children: Vec<CircuitRc>) -> Result<Self> {
+        Self::try_new(children[0].clone(), (*self.spec).clone(), self.info().name)
     }
 
     fn eval_tensors(&self, tensors: &[Tensor]) -> Result<Tensor> {
@@ -850,7 +804,7 @@ impl CircuitNodeAutoName for Rearrange {
     const PRIORITY: OperatorPriority = OperatorPriority::PostFix {};
 
     fn auto_name(&self) -> Option<Name> {
-        child_name_with_maybe_paren(&Self::PRIORITY, self.node.clone())
+        child_name_with_maybe_paren(&Self::PRIORITY, self.node().clone())
             .map(|n| (n.string() + " rearrange").into())
     }
 }
@@ -892,12 +846,24 @@ impl Rearrange {
         )
     }
 
+    #[getter]
+    pub fn node(&self) -> CircuitRc {
+        self.info.children[0].clone()
+    }
+
     pub fn conform_to_input_shape_spec(&self) -> RearrangeSpec {
-        self.spec.conform_to_input_shape(self.node.shape()).unwrap()
+        self.spec
+            .conform_to_input_shape(self.node().shape())
+            .unwrap()
     }
 
     pub fn conform_to_input_shape(&self) -> Self {
-        Rearrange::try_new(self.node.clone(), self.conform_to_input_shape_spec(), None).unwrap()
+        Rearrange::try_new(
+            self.node().clone(),
+            self.conform_to_input_shape_spec(),
+            None,
+        )
+        .unwrap()
     }
 }
 
@@ -909,8 +875,6 @@ impl Rearrange {
 #[pyclass(extends=PyCircuitBase)]
 #[derive(Clone)]
 pub struct Index {
-    #[pyo3(get)]
-    pub node: CircuitRc,
     #[pyo3(get, name = "idx")]
     pub index: TensorIndex,
     info: CachedCircuitInfo,
@@ -953,12 +917,14 @@ impl Index {
 
         index.validate(&node.info().shape)?;
 
-        let mut out = Self {
-            node,
+        let out = Self {
+            info: CachedCircuitInfo::incomplete(
+                name,
+                index.apply_to_shape(&node.info().shape),
+                vec![node],
+            ),
             index,
-            info: Default::default(),
         };
-        out.info.name = name;
 
         out.initial_init_info()
     }
@@ -967,19 +933,22 @@ impl Index {
 circuit_node_extra_impl!(Index, self_hash_default);
 
 impl CircuitNodeComputeInfoImpl for Index {
-    fn compute_shape(&self) -> Shape {
-        self.index.apply_to_shape(&self.node.info().shape)
-    }
-
-    fn device_dtype_extra(&self) -> Box<dyn Iterator<Item = TorchDeviceDtypeOp> + '_> {
-        Box::new(self.index.0.iter().filter_map(|x| match x {
-            TensorAxisIndex::Tensor(tensor) if !tensor.shape().is_empty() => {
-                let mut out: TorchDeviceDtypeOp = TorchDeviceDtype::from_tensor(tensor).into();
-                out.dtype = None;
-                Some(out)
-            }
-            _ => None,
-        }))
+    fn device_dtype_extra(&self) -> Option<Vec<TorchDeviceDtypeOp>> {
+        Some(
+            self.index
+                .0
+                .iter()
+                .filter_map(|x| match x {
+                    TensorAxisIndex::Tensor(tensor) if !tensor.shape().is_empty() => {
+                        let mut out: TorchDeviceDtypeOp =
+                            TorchDeviceDtype::from_tensor(tensor).into();
+                        out.dtype = None;
+                        Some(out)
+                    }
+                    _ => None,
+                })
+                .collect(),
+        )
     }
 }
 
@@ -994,7 +963,7 @@ impl CircuitNode for Index {
 
     fn child_axis_map(&self) -> Vec<Vec<Option<usize>>> {
         let mut cur: i32 = -1;
-        let result = vec![zip(&self.node.info().shape, &self.index.0)
+        let result = vec![zip(&self.node().info().shape, &self.index.0)
             .map(|(l, idx)| {
                 if !matches!(idx, TensorAxisIndex::Single(_)) {
                     cur += 1;
@@ -1009,19 +978,8 @@ impl CircuitNode for Index {
         result
     }
 
-    fn children(&self) -> Box<dyn Iterator<Item = CircuitRc> + '_> {
-        Box::new(std::iter::once(self.node.clone()))
-    }
-
-    fn map_children_enumerate<F>(&self, mut f: F) -> Result<Self>
-    where
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>,
-    {
-        Self::try_new(
-            f(0, self.node.clone())?,
-            self.index.clone(),
-            self.info().name,
-        )
+    fn _replace_children(&self, children: Vec<CircuitRc>) -> Result<Self> {
+        Self::try_new(children[0].clone(), self.index.clone(), self.info().name)
     }
 
     fn eval_tensors(&self, tensors: &[Tensor]) -> Result<Tensor> {
@@ -1045,7 +1003,7 @@ impl CircuitNodeAutoName for Index {
     const PRIORITY: OperatorPriority = OperatorPriority::PostFix {};
 
     fn auto_name(&self) -> Option<Name> {
-        child_name_with_maybe_paren(&Self::PRIORITY, self.node.clone())
+        child_name_with_maybe_paren(&Self::PRIORITY, self.node().clone())
             .map(|n| (n.string() + " idx").into())
     }
 }
@@ -1063,12 +1021,17 @@ impl Index {
         Ok(out.into_init())
     }
 
+    #[getter]
+    pub fn node(&self) -> CircuitRc {
+        self.info.children[0].clone()
+    }
+
     // TODO: add support for synchronized index construction like python
     pub fn slice_edges_to_none(&self) -> Self {
         Index::try_new(
-            self.node.clone(),
+            self.node(),
             TensorIndex(
-                zip(&self.index.0, &self.node.info().shape)
+                zip(&self.index.0, &self.node().info().shape)
                     .map(|(ax, l)| match ax {
                         TensorAxisIndex::Slice(slice) => TensorAxisIndex::Slice(Slice {
                             start: if slice.start_u(*l) == 0 {
@@ -1246,7 +1209,7 @@ impl Index {
 #[pyclass(extends=PyCircuitBase)]
 #[derive(Clone)]
 pub struct Concat {
-    pub nodes: Vec<CircuitRc>,
+    #[pyo3(get)]
     pub axis: usize,
     info: CachedCircuitInfo,
 }
@@ -1275,12 +1238,13 @@ impl Concat {
             }
         }
 
-        let mut out = Self {
-            nodes,
+        let mut shape = nodes[0].info().shape.clone();
+        shape[axis] = nodes.iter().map(|n| n.info().shape[axis]).sum();
+
+        let out = Self {
             axis,
-            info: Default::default(),
+            info: CachedCircuitInfo::incomplete(name, shape, nodes),
         };
-        out.info.name = name;
         out.initial_init_info()
     }
 
@@ -1301,13 +1265,7 @@ impl Concat {
 
 circuit_node_extra_impl!(Concat, self_hash_default);
 
-impl CircuitNodeComputeInfoImpl for Concat {
-    fn compute_shape(&self) -> Shape {
-        let mut shape = self.nodes[0].info().shape.clone();
-        shape[self.axis] = self.nodes.iter().map(|n| n.info().shape[self.axis]).sum();
-        shape
-    }
-}
+impl CircuitNodeComputeInfoImpl for Concat {}
 
 impl CircuitNodeHashItems for Concat {
     fn compute_hash_non_name_non_children(&self, hasher: &mut blake3::Hasher) {
@@ -1318,23 +1276,8 @@ impl CircuitNodeHashItems for Concat {
 impl CircuitNode for Concat {
     circuit_node_auto_impl!("f2684583-c215-4f67-825e-6e4e51091ca7");
 
-    fn children(&self) -> Box<dyn Iterator<Item = CircuitRc> + '_> {
-        Box::new(self.nodes.iter().cloned())
-    }
-
-    fn map_children_enumerate<F>(&self, mut f: F) -> Result<Self>
-    where
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>,
-    {
-        Self::try_new(
-            self.nodes
-                .iter()
-                .enumerate()
-                .map(move |(i, circ)| f(i, circ.clone()))
-                .collect::<Result<Vec<_>, _>>()?,
-            self.axis,
-            self.info().name,
-        )
+    fn _replace_children(&self, children: Vec<CircuitRc>) -> Result<Self> {
+        Self::try_new(children, self.axis, self.info().name)
     }
 
     fn child_axis_map(&self) -> Vec<Vec<Option<usize>>> {
@@ -1345,7 +1288,7 @@ impl CircuitNode for Concat {
                     false => Some(i),
                 })
                 .collect();
-            self.nodes.len()
+            self.num_children()
         ]
     }
 
@@ -1390,21 +1333,8 @@ impl Concat {
         Self::new_signed_axis(nodes, axis, name).map(|x| x.into_init())
     }
 
-    #[getter]
-    fn nodes(&self) -> Vec<CircuitRc> {
-        self.nodes.to_vec()
-    }
-
-    #[getter]
-    fn axis(&self) -> usize {
-        self.axis
-    }
-
     pub fn get_sizes_at_axis(&self) -> Vec<usize> {
-        self.nodes
-            .iter()
-            .map(|x| x.info().shape[self.axis])
-            .collect()
+        self.children().map(|x| x.info().shape[self.axis]).collect()
     }
 
     #[staticmethod]
@@ -1426,6 +1356,11 @@ impl Concat {
             .context("failed to unsqueeze for stack (axis invalid?)")?;
         Self::try_new(to_cat, axis, name).context("failed to concat for stack after unsqueeze")
     }
+
+    #[getter]
+    pub fn nodes(&self) -> Vec<CircuitRc> {
+        self.info().children.clone()
+    }
 }
 
 /// Scatter is equivalent to:
@@ -1437,8 +1372,6 @@ impl Concat {
 #[pyclass(extends=PyCircuitBase)]
 #[derive(Clone)]
 pub struct Scatter {
-    #[pyo3(get)]
-    pub node: CircuitRc,
     #[pyo3(get, name = "idx")]
     pub index: TensorIndex,
     info: CachedCircuitInfo,
@@ -1466,13 +1399,10 @@ impl Scatter {
             .into());
         }
 
-        let mut out = Self {
-            node,
+        let out = Self {
             index,
-            info: Default::default(),
+            info: CachedCircuitInfo::incomplete(name, shape, vec![node]),
         };
-        out.info.name = name;
-        out.info.shape = shape;
         // todo check not too many axes / oob
         out.initial_init_info()
     }
@@ -1480,11 +1410,7 @@ impl Scatter {
 
 circuit_node_extra_impl!(Scatter, self_hash_default);
 
-impl CircuitNodeComputeInfoImpl for Scatter {
-    fn compute_shape(&self) -> Shape {
-        self.info().shape.clone()
-    }
-}
+impl CircuitNodeComputeInfoImpl for Scatter {}
 
 impl CircuitNodeHashItems for Scatter {
     fn compute_hash_non_name_non_children(&self, hasher: &mut blake3::Hasher) {
@@ -1498,10 +1424,6 @@ impl CircuitNodeHashItems for Scatter {
 impl CircuitNode for Scatter {
     circuit_node_auto_impl!("50cce6d3-457c-4f52-9a23-1903bdc76533");
 
-    fn children(&self) -> Box<dyn Iterator<Item = CircuitRc> + '_> {
-        Box::new(std::iter::once(self.node.clone()))
-    }
-
     fn child_axis_map(&self) -> Vec<Vec<Option<usize>>> {
         let mut cur: i32 = -1;
         vec![zip(&self.info().shape, &self.index.0)
@@ -1519,12 +1441,9 @@ impl CircuitNode for Scatter {
             .collect()]
     }
 
-    fn map_children_enumerate<F>(&self, mut f: F) -> Result<Self>
-    where
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>,
-    {
+    fn _replace_children(&self, children: Vec<CircuitRc>) -> Result<Self> {
         Self::try_new(
-            f(0, self.node.clone())?,
+            children[0].clone(),
             self.index.clone(),
             self.info().shape.clone(),
             self.info().name,
@@ -1550,7 +1469,7 @@ impl CircuitNodeAutoName for Scatter {
     const PRIORITY: OperatorPriority = OperatorPriority::PostFix {};
 
     fn auto_name(&self) -> Option<Name> {
-        child_name_with_maybe_paren(&Self::PRIORITY, self.node.clone())
+        child_name_with_maybe_paren(&Self::PRIORITY, self.node())
             .map(|n| (n.string() + " scatter").into())
     }
 }
@@ -1569,8 +1488,13 @@ impl Scatter {
         Ok(out.into_init())
     }
 
+    #[getter]
+    pub fn node(&self) -> CircuitRc {
+        self.info.children[0].clone()
+    }
+
     pub fn is_identity(&self) -> bool {
-        self.info().shape[..] == self.node.info().shape[..] && self.index.all_uslices().is_some()
+        self.info().shape[..] == self.node().info().shape[..] && self.index.all_uslices().is_some()
     }
 }
 
@@ -1624,15 +1548,10 @@ pub fn flat_concat_back(circuits: Vec<CircuitRc>) -> Result<(Concat, Vec<Circuit
 #[pyclass(unsendable, extends=PyCircuitBase)]
 #[derive(Clone)]
 pub struct Conv {
-    #[pyo3(get)]
-    pub input: CircuitRc,
-    #[pyo3(get)]
-    pub filter: CircuitRc,
     #[pyo3(get)]
     pub stride: Vec<usize>,
     #[pyo3(get)]
     pub padding: Vec<(usize, usize)>,
-
     info: CachedCircuitInfo,
 }
 
@@ -1658,41 +1577,34 @@ impl Conv {
             });
         }
 
-        let mut out = Self {
-            input,
-            filter,
+        let shape = Self::try_compute_shape(&input, &filter, &stride, &padding)?;
+
+        let out = Self {
             stride,
             padding,
-            info: Default::default(),
+            info: CachedCircuitInfo::incomplete(name, shape, vec![input, filter]),
         };
-        out.info.name = name;
-        out.info.shape = out.try_compute_shape()?;
         // todo check not too many axes / oob
         out.initial_init_info()
     }
 
-    pub fn try_compute_shape(&self) -> Result<Shape> {
-        let dims = self.dims();
+    fn try_compute_shape(
+        input: &CircuitRc,
+        filter: &CircuitRc,
+        stride: &[usize],
+        padding: &[(usize, usize)],
+    ) -> Result<Shape> {
+        let dims = stride.len();
         let input_batch_shape =
-            if let Some(input_batch_rank) = self.input.info().rank().checked_sub(dims + 1) {
-                Ok::<Shape, ConstructError>(
-                    self.input.info().shape[..input_batch_rank]
-                        .iter()
-                        .cloned()
-                        .collect(),
-                )
+            if let Some(input_batch_rank) = input.info().rank().checked_sub(dims + 1) {
+                Ok::<Shape, ConstructError>(input.info().shape[..input_batch_rank].to_smallvec())
             } else {
                 bail!(ConstructError::ConvInputWrongShape {});
             }?;
 
         let filter_batch_shape =
-            if let Some(filter_batch_rank) = self.filter.info().rank().checked_sub(dims + 2) {
-                Ok::<Shape, ConstructError>(
-                    self.input.info().shape[..filter_batch_rank]
-                        .iter()
-                        .cloned()
-                        .collect(),
-                )
+            if let Some(filter_batch_rank) = filter.info().rank().checked_sub(dims + 2) {
+                Ok::<Shape, ConstructError>(input.info().shape[..filter_batch_rank].to_smallvec())
             } else {
                 bail!(ConstructError::ConvFilterWrongShape {});
             }?;
@@ -1708,33 +1620,30 @@ impl Conv {
                 ),
             });
         }
-        let out_channels = self.filter.info().shape[filter_batch_shape.len()];
-        let in_channels = self.filter.info().shape[self.filter.info().rank() - 1];
-        if in_channels != self.input.info().shape[self.input.info().rank() - 1] {
+        let out_channels = filter.info().shape[filter_batch_shape.len()];
+        let in_channels = filter.info().shape[filter.info().rank() - 1];
+        if in_channels != input.info().shape[input.info().rank() - 1] {
             bail!(ConstructError::ConvInputFilterDifferentNumInputChannels {
                 filter: in_channels,
-                input: self.input.info().shape[self.input.info().rank() - 1],
+                input: input.info().shape[input.info().rank() - 1],
             });
         }
         let input_conv_shape =
-            &self.input.info().shape[input_batch_shape.len()..input_batch_shape.len() + dims];
-        let filter_kernel_shape = &self.filter.info().shape
-            [self.filter.info().rank() - dims - 1..self.filter.info().rank() - 1];
+            &input.info().shape[input_batch_shape.len()..input_batch_shape.len() + dims];
+        let filter_kernel_shape =
+            &filter.info().shape[filter.info().rank() - dims - 1..filter.info().rank() - 1];
         let input_conv_shape_padded_minus_edges: Shape =
-            zip(input_conv_shape, zip(&self.padding, filter_kernel_shape))
+            zip(input_conv_shape, zip(padding, filter_kernel_shape))
                 .map(|(input_l, (padding, kernel_l))| {
                     input_l - (kernel_l - 1) + padding.0 + padding.1
                 })
                 .collect();
-        let new_conv_shape: Shape = zip(&input_conv_shape_padded_minus_edges, &self.stride)
+        let new_conv_shape: Shape = zip(&input_conv_shape_padded_minus_edges, stride)
             .map(|(i, s)| {
                 if i % s != 0 {
                     Err(ConstructError::ConvStrideMustDivide {
-                        shape: input_conv_shape_padded_minus_edges
-                            .iter()
-                            .cloned()
-                            .collect(),
-                        stride: self.stride.clone(),
+                        shape: input_conv_shape_padded_minus_edges.clone(),
+                        stride: stride.to_vec(),
                     })
                     .context("hi")
                 } else {
@@ -1753,11 +1662,7 @@ impl Conv {
 
 circuit_node_extra_impl!(Conv, self_hash_default);
 
-impl CircuitNodeComputeInfoImpl for Conv {
-    fn compute_shape(&self) -> Shape {
-        self.info().shape.clone()
-    }
-}
+impl CircuitNodeComputeInfoImpl for Conv {}
 impl CircuitNodeHashItems for Conv {
     fn compute_hash_non_name_non_children(&self, hasher: &mut blake3::Hasher) {
         // this is ok because all looped over items are fixed len and we follow this with uuid (from children hash)
@@ -1772,10 +1677,6 @@ impl CircuitNodeHashItems for Conv {
 impl CircuitNode for Conv {
     circuit_node_auto_impl!("a329465f-a18f-4313-a2a6-51d8ebad7dd6");
 
-    fn children(&self) -> Box<dyn Iterator<Item = CircuitRc> + '_> {
-        Box::new([self.input.clone(), self.filter.clone()].into_iter())
-    }
-
     fn child_axis_map(&self) -> Vec<Vec<Option<usize>>> {
         // todo: this doesn't state all available axis linkings
         vec![
@@ -1783,21 +1684,18 @@ impl CircuitNode for Conv {
                 .map(Some)
                 .chain(vec![
                     None;
-                    self.input.info().rank()
+                    self.input().info().rank()
                         - self.input_batch_shape().len()
                 ])
                 .collect(),
-            vec![None; self.filter.info().rank()],
+            vec![None; self.filter().info().rank()],
         ]
     }
 
-    fn map_children_enumerate<F>(&self, mut f: F) -> Result<Self>
-    where
-        F: FnMut(usize, CircuitRc) -> Result<CircuitRc>,
-    {
+    fn _replace_children(&self, children: Vec<CircuitRc>) -> Result<Self> {
         Self::try_new(
-            f(0, self.input.clone())?,
-            f(1, self.filter.clone())?,
+            children[0].clone(),
+            children[1].clone(),
             self.stride.clone(),
             self.padding.clone(),
             self.info().name,
@@ -1824,8 +1722,8 @@ impl CircuitNodeAutoName for Conv {
     const PRIORITY: OperatorPriority = OperatorPriority::Function {};
 
     fn auto_name(&self) -> Option<Name> {
-        let input_name = self.input.info().name?;
-        let filter_name = self.filter.info().name?;
+        let input_name = self.input().info().name?;
+        let filter_name = self.filter().info().name?;
         Some(format!("Conv({input_name}, {filter_name})").into())
     }
 }
@@ -1867,9 +1765,17 @@ impl Conv {
         self.stride.len()
     }
     pub fn input_batch_shape(&self) -> Shape {
-        self.input.info().shape[..self.info().rank() - 1 - self.dims()]
+        self.input().info().shape[..self.info().rank() - 1 - self.dims()]
             .iter()
             .cloned()
             .collect()
     }
+    #[getter]
+    pub fn input(&self) -> CircuitRc {
+        self.info.children[0].clone()
+    }
+    #[getter]
+    pub fn filter(&self) -> CircuitRc {
+        self.info.children[1].clone()
+    }
 }
