@@ -116,8 +116,8 @@ macro_rules! circuit_node_self_only_hash_default_impl {
 pub trait CircuitNode: CircuitNodeInit + CircuitNodeSelfOnlyHash + Clone {
     // ==== implementable section ===
     //
-    // NOTE: ALL FNS IN THIS SECTION *MUST* BE COPIED TO THE CIRCUIT NODE UNION IMPL!
-    // If you add something here with a default impl, write a new impl for circuit node union!
+    // NOTE: ALL FNS IN THIS SECTION *MUST* BE COPIED TO THE MACRO AND CircuitRc impl
+    // If you add something here with a default impl, write a new impl for macro!
     // (up until default only section)
     //
     // we could enforce this sort of stuff with some proc macros, but seems like overkill atm.
@@ -162,7 +162,7 @@ pub trait CircuitNode: CircuitNodeInit + CircuitNodeSelfOnlyHash + Clone {
 
     // ==== default only section ===
     // FUNCTIONS BELOW HERE *shouldn't* be overridden by implementors!
-    // (if you do implement, this won't be picked up on by CircuitNodeDefer (union types etc)!)
+    // (if you do implement, this won't be picked up on by union types & CircuitRc
     #[inline]
     fn compute_flags_default(&self) -> CircuitFlags {
         let mut result: CircuitFlags = CircuitFlags::all_true();
@@ -842,6 +842,10 @@ macro_rules! define_circuit_union_impl {
                 [<$name:snake _map>]!(self; node => node.info())
             }
 
+            fn num_free_children(&self) -> usize {
+                [<$name:snake _map>]!(self; node => node.num_free_children())
+            }
+
             fn _replace_children(&self, children: Vec<CircuitRc>) -> Result<Self> {
                 [<$name:snake _map_construct>]!(self; (node, construct) => Ok(construct(node._replace_children(children)?)))
             }
@@ -858,7 +862,6 @@ macro_rules! define_circuit_union_impl {
                 [<$name:snake _map>]!(self; node => node.eval_tensors(tensors))
             }
 
-            // TODO: can return self for Circuit.c()
             fn c(self) -> Circuit { to_circuit_maybe_short_circuit!($name, self, [<$name:snake _map>]!(self; node => node.c())) }
             fn rc(self) -> CircuitRc { [<$name:snake _map>]!(self; node => node.rc()) }
             fn get_autoname(&self) -> Result<Option<Name>> { [<$name:snake _map>]!(self; node => node.get_autoname()) }
@@ -1521,6 +1524,10 @@ impl CircuitNode for CircuitRc {
         self.0.info()
     }
 
+    fn num_free_children(&self) -> usize {
+        self.0.num_free_children()
+    }
+
     fn child_axis_map(&self) -> Vec<Vec<Option<usize>>> {
         self.0.child_axis_map()
     }
