@@ -28,7 +28,7 @@ use uuid::{uuid, Uuid};
 
 use crate::{
     circuit_node_auto_impl, circuit_node_extra_impl,
-    circuit_node_private::{CircuitNodeHashItems, CircuitNodePrivate, CircuitNodeSetNonHashInfo},
+    circuit_node_private::{CircuitNodeHashItems, CircuitNodeSetNonHashInfo},
     circuit_utils::OperatorPriority,
     deep_map, deep_map_op, deep_map_op_context_preorder_stoppable, deep_map_pre_new_children,
     deep_map_preorder_unwrap,
