@@ -21,7 +21,10 @@ pub fn elim_empty_module(module: &Module) -> Option<CircuitRc> {
 
 #[pyfunction]
 pub fn elim_no_input_module(module: &Module) -> Option<CircuitRc> {
-    module.nodes.is_empty().then(|| module.spec.circuit.clone())
+    module
+        .args_slice()
+        .is_empty()
+        .then(|| module.spec.circuit.clone())
 }
 
 #[pyfunction]
@@ -237,7 +240,7 @@ pub fn fuse_concat_modules(
             let dif = x.info().rank() - spec.circuit.info().rank();
             match dif {
                 0 => Ok(Module::new(
-                    zip(&x.nodes, &x.spec.arg_specs)
+                    zip(x.args(), &x.spec.arg_specs)
                         .map(|(n, s)| {
                             if s.batchable {
                                 n.unsqueeze(vec![0], None).unwrap().rc()
@@ -263,7 +266,7 @@ pub fn fuse_concat_modules(
             x.batchable
                 || modules_one_batch
                     .iter()
-                    .all(|node| node.nodes[i] == modules_one_batch[0].nodes[i])
+                    .all(|node| node.args_slice()[i] == modules_one_batch[0].args_slice()[i])
         })
     {
         bail!(anyhow!(
@@ -277,13 +280,13 @@ pub fn fuse_concat_modules(
                     Concat::nrc(
                         modules_one_batch
                             .iter()
-                            .map(|m| m.nodes[i].clone())
+                            .map(|m| m.args_slice()[i].clone())
                             .collect(),
                         0,
                         None,
                     )
                 } else {
-                    modules_one_batch[0].nodes[i].clone()
+                    modules_one_batch[0].args_slice()[i].clone()
                 }
             })
             .collect(),
@@ -298,7 +301,8 @@ pub fn fuse_concat_modules(
             .map(|x| x.info().shape[0])
             .collect(),
     )
-    .nodes;
+    .children()
+    .collect();
     let splitted_unsqueezed: Vec<CircuitRc> = (0..modules.len())
         .map(|i| {
             if modules[i].info().rank() == pure_rank {
