@@ -1060,7 +1060,8 @@ impl IterativeMatcher {
                         first,
                         rest.map(|x| {
                             if must_be_sub {
-                                x.filter(
+                                restrict(
+                                    x,
                                     false,
                                     Some(1),
                                     None,
@@ -1177,49 +1178,6 @@ macro_rules! dup_functions {
 #[apply(dup_functions)]
 #[self_id(self_)]
 impl IterativeMatcher {
-    #[pyo3(signature=(
-        term_if_matches = false,
-        start_depth = None,
-        end_depth = None,
-        term_early_at = MatcherFromPyBase::Always(false).into()
-    ))]
-    pub fn filter(
-        &self,
-        term_if_matches: bool,
-        start_depth: Option<u32>,
-        end_depth: Option<u32>,
-        term_early_at: MatcherRc,
-    ) -> IterativeMatcher {
-        // DEPRECATED: use restrict instead
-        restrict(
-            self_.clone().rc(),
-            term_if_matches,
-            start_depth,
-            end_depth,
-            term_early_at,
-        )
-    }
-
-    #[pyo3(signature=(
-        term_if_matches = false,
-        depth_slice = Slice::IDENT,
-        term_early_at = MatcherFromPyBase::Always(false).into()
-    ))]
-    pub fn filter_sl(
-        &self,
-        term_if_matches: bool,
-        depth_slice: Slice,
-        term_early_at: MatcherRc,
-    ) -> Result<IterativeMatcher> {
-        // DEPRECATED: use restrict_sl instead
-        restrict_sl(
-            self_.clone().rc(),
-            term_if_matches,
-            depth_slice,
-            term_early_at,
-        )
-    }
-
     #[pyo3(signature=(*rest, must_be_sub = false))]
     pub fn chain(&self, rest: Vec<IterativeMatcherRc>, must_be_sub: bool) -> IterativeMatcher {
         // TODO: flatten
@@ -1328,7 +1286,8 @@ impl IterativeMatcher {
         transform,
         cache_transform = Updater::default().cache_transform,
         cache_update = Updater::default().cache_update,
-        fancy_validate = Updater::default().default_fancy_validate
+        fancy_validate = Updater::default().default_fancy_validate,
+        assert_any_found = Updater::default().default_assert_any_found
     ))]
     pub fn update(
         &self,
@@ -1337,11 +1296,13 @@ impl IterativeMatcher {
         cache_transform: bool,
         cache_update: bool,
         fancy_validate: bool,
+        assert_any_found: bool,
     ) -> Result<CircuitRc> {
-        Updater::new(transform, cache_transform, cache_update, false).update(
+        Updater::new(transform, cache_transform, cache_update, false, false).update(
             circuit,
             self_.crc(),
             Some(fancy_validate),
+            Some(assert_any_found),
         )
     }
 
@@ -1349,7 +1310,8 @@ impl IterativeMatcher {
         transform,
         cache_transform = Updater::default().cache_transform,
         cache_update = Updater::default().cache_update,
-        default_fancy_validate = Updater::default().default_fancy_validate
+        default_fancy_validate = Updater::default().default_fancy_validate,
+        default_assert_any_found = Updater::default().default_assert_any_found
     ))]
     pub fn updater(
         &self,
@@ -1357,12 +1319,14 @@ impl IterativeMatcher {
         cache_transform: bool,
         cache_update: bool,
         default_fancy_validate: bool,
+        default_assert_any_found: bool,
     ) -> BoundUpdater {
         Updater::new(
             transform,
             cache_transform,
             cache_update,
             default_fancy_validate,
+            default_assert_any_found,
         )
         .bind(self_.crc())
     }
@@ -1452,7 +1416,8 @@ pub fn restrict_sl(
     depth_slice: Slice,
     term_early_at: MatcherRc,
 ) -> Result<IterativeMatcher> {
-    Ok(matcher.filter(
+    Ok(restrict(
+        matcher,
         term_if_matches,
         flip_op_result(depth_slice.start.map(|x| x.try_into()))?,
         flip_op_result(depth_slice.stop.map(|x| x.try_into()))?,
