@@ -33,7 +33,7 @@ pub fn rewrite_cum_to_circuit_of_cum(
     namer: Option<PyObject>,
     on_sub_cumulant_fn: Option<PyObject>,
 ) -> CircResult {
-    if cumulant.nodes.len() == 0 {
+    if cumulant.num_children() == 0 {
         return Ok(Scalar::nrc(1., sv![], cumulant.info().name));
     }
 
@@ -79,7 +79,7 @@ pub fn rewrite_cum_to_circuit_of_cum(
 
             let new_spec = RearrangeSpec::new(new_input_ints, new_output_ints, new_int_sizes)?;
             Rearrange::nrc(
-                on_sub_cumulant(replace_node(cumulant, n.node.clone(), node_position))?,
+                on_sub_cumulant(replace_node(cumulant, n.node().clone(), node_position))?,
                 new_spec,
                 original_cumulant_name,
             )
@@ -90,21 +90,20 @@ pub fn rewrite_cum_to_circuit_of_cum(
                 .collect();
             new_index.append(&mut n.index.clone().0);
             Index::nrc(
-                on_sub_cumulant(replace_node(cumulant, n.node.clone(), node_position))?,
+                on_sub_cumulant(replace_node(cumulant, n.node().clone(), node_position))?,
                 TensorIndex(new_index),
                 original_cumulant_name,
             )
         }
         Circuit::Tag(n) => Tag::nrc(
-            on_sub_cumulant(replace_node(cumulant, n.node.clone(), node_position))?,
+            on_sub_cumulant(replace_node(cumulant, n.node().clone(), node_position))?,
             n.uuid,
             original_cumulant_name,
         ),
         Circuit::Concat(n) => {
             let axis = start_axis + n.axis;
             Concat::nrc(
-                n.nodes
-                    .iter()
+                n.children()
                     .map(|c| replace_node(cumulant.clone(), c.clone(), node_position))
                     .map(on_sub_cumulant)
                     .collect::<Result<Vec<CircuitRc>>>()?,
@@ -118,9 +117,8 @@ pub fn rewrite_cum_to_circuit_of_cum(
 
             Add::nrc(
                 explicit_add
-                    .nodes
-                    .iter()
-                    .map(|c| replace_node(cumulant.clone(), c.clone(), node_position))
+                    .children()
+                    .map(|c| replace_node(cumulant.clone(), c, node_position))
                     .map(on_sub_cumulant)
                     .collect::<Result<Vec<CircuitRc>>>()?,
                 original_cumulant_name,
@@ -142,7 +140,7 @@ pub fn rewrite_cum_to_circuit_of_cum(
             bail!(CumulantRewriteError::UnimplementedType { node })
         }
         Circuit::Array(_) | Circuit::Scalar(_) | Circuit::Cumulant(_) => {
-            if cumulant.nodes.len() == 1 {
+            if cumulant.num_children() == 1 {
                 node.rename(cumulant.info().name).rc()
             } else {
                 Scalar::nrc(0., cumulant.shape().clone(), cumulant.info().name)
@@ -158,7 +156,7 @@ pub fn rewrite_cum_to_circuit_of_cum(
 
 fn replace_node(cumulant: Cumulant, node: CircuitRc, position: usize) -> Cumulant {
     let new_name = cumulant.info().name;
-    let mut new_nodes = cumulant.nodes;
+    let mut new_nodes = cumulant.children_sl().to_vec();
     new_nodes[position] = node;
     Cumulant::new(new_nodes, new_name)
 }
@@ -170,14 +168,13 @@ fn cum_of_einsum_to_einsum_of_cum_raw(
     just_fully_touching_partitions: bool,
     on_sub_cumulant: impl Fn(Cumulant) -> CircResult,
 ) -> CircResult {
-    let einsum = cumulant.nodes[position].as_einsum_unwrap();
+    let einsum = cumulant.children_sl()[position].as_einsum_unwrap();
 
     let other_nodes: Vec<CircuitRc> = cumulant
-        .nodes
-        .iter()
+        .children()
         .enumerate()
         .filter(|(i, _)| *i != position)
-        .map(|(_, c)| c.clone())
+        .map(|(_, c)| c)
         .collect();
     let other_rank = cumulant.rank() - einsum.rank();
     let other_nums: Vec<u8> =
@@ -189,17 +186,17 @@ fn cum_of_einsum_to_einsum_of_cum_raw(
     ]
     .concat();
 
-    let count = einsum.args.len() + other_nodes.len();
+    let count = einsum.num_children() + other_nodes.len();
 
     let mut to_sum: Vec<CircuitRc> = vec![];
 
     for p_other in partition_into_blocks(
-        zip(einsum.args.len()..count, other_nodes.clone()).collect(),
+        zip(einsum.num_children()..count, other_nodes.clone()).collect(),
         0,
-        einsum.args.len(),
+        einsum.num_children(),
     ) {
         for p_mine in partition_into_blocks(
-            einsum.input_circuits().enumerate().collect(),
+            einsum.children().enumerate().collect(),
             p_other.len(),
             if just_fully_touching_partitions {
                 p_other.len()
@@ -238,7 +235,10 @@ fn cum_of_einsum_to_einsum_of_cum_raw(
                 let (prod, _new_cums) = kappa_term(fused_per, &on_sub_cumulant)?;
 
                 let operated_on_prod = Einsum::try_new(
-                    vec![(prod.rc(), einsum.input_axes().flatten().cloned().collect())],
+                    vec![(
+                        prod.rc(),
+                        einsum.in_axes.iter().flatten().cloned().collect(),
+                    )],
                     out_nums.clone().into(),
                     Some("perm".into()), /* name=Cumulant.get_name(*new_cums, getter=namer) + " perm",  # naming to ensure no duplicates */
                 )?; // TODO: better name
@@ -298,7 +298,7 @@ fn kappa_term(
             None,
             Some(dim_permutation_for_circuits(
                 args,
-                new_cums.iter().map(|c| c.clone().nodes).collect(),
+                new_cums.iter().map(|c| c.children_sl()).collect(),
                 count,
             )),
         ),
@@ -354,12 +354,12 @@ fn find_position_and_axis_of_start(
 ) -> Result<(usize, usize)> {
     let mut node_position = -1;
     let mut start_axis = 0;
-    for i in 0..cumulant.nodes.len() {
-        if cumulant.nodes[i] == *node {
+    for i in 0..cumulant.num_children() {
+        if cumulant.children_sl()[i] == *node {
             node_position = i as i32;
             break;
         } else {
-            start_axis += cumulant.nodes[i].rank();
+            start_axis += cumulant.children_sl()[i].rank();
         }
     }
     if node_position == -1 {
