@@ -435,15 +435,13 @@ impl NestFullInfo for NestEinsumsFullInfo {
             .collect();
 
         let new_args = einsum
-            .args
-            .clone()
-            .into_iter()
+            .args()
             .zip(rec_on_children)
             .flat_map(|((node, ints), (_, named_idxs, _))| {
                 if named_idxs.is_some() {
-                    node.as_einsum().unwrap().args.clone()
+                    node.as_einsum().unwrap().args_cloned()
                 } else {
-                    vec![(node, ints)]
+                    vec![(node, ints.clone())]
                 }
             })
             .collect();
@@ -457,7 +455,12 @@ impl NestFullInfo for NestEinsumsFullInfo {
         flat: &Self::CircuitType,
         specs: Vec<NestIdxsInfo<Self>>,
     ) -> Result<Self::CircuitType> {
-        nest_flat_einsum_strict_rec(&flat.args, flat.out_axes.clone(), specs, flat.info().name)
+        nest_flat_einsum_strict_rec(
+            &flat.args_cloned(),
+            flat.out_axes.clone(),
+            specs,
+            flat.info().name,
+        )
     }
 }
 impl NestFullInfo for NestAddsFullInfo {
@@ -475,14 +478,13 @@ impl NestFullInfo for NestAddsFullInfo {
         add: Add,
     ) -> FlattenImplRec<Self> {
         let new_operands: Vec<CircuitRc> = add
-            .nodes
-            .iter()
+            .children()
             .zip(rec_on_children)
             .flat_map(|(node, (_, named_idxs, _))| {
                 if named_idxs.is_some() {
-                    node.as_add().unwrap().nodes.clone()
+                    node.as_add().unwrap().children_sl().to_vec()
                 } else {
-                    vec![node.clone()]
+                    vec![node]
                 }
             })
             .collect();
@@ -496,7 +498,7 @@ impl NestFullInfo for NestAddsFullInfo {
         specs: Vec<NestIdxsInfo<Self>>,
     ) -> Result<Self::CircuitType> {
         Ok(nest_flat_add_strict_rec(
-            &flat.nodes,
+            flat.children_sl(),
             specs,
             flat.info().name,
         ))
