@@ -1254,16 +1254,18 @@ impl TorchDeviceDtype {
         })
     }
     pub fn size(&self) -> usize {
-        let map = HashMap::from_iter([
-            (TorchDtype::float32, 4),
-            (TorchDtype::float64, 8),
-            (TorchDtype::float16, 2),
-            (TorchDtype::int64, 8),
-            (TorchDtype::int32, 4),
-            (TorchDtype::int16, 2),
-            (TorchDtype::int8, 1),
-        ]);
-        map[&self.dtype]
+        match self.dtype {
+            TorchDtype::float32 => 4,
+            TorchDtype::float64 => 8,
+            TorchDtype::float16 => 2,
+            TorchDtype::bfloat16 => 2,
+            TorchDtype::int64 => 8,
+            TorchDtype::int32 => 4,
+            TorchDtype::int16 => 2,
+            TorchDtype::int8 => 1,
+            TorchDtype::uint8 => 1,
+            TorchDtype::bool => 1,
+        }
     }
 }
 
