@@ -4,29 +4,8 @@ import functools
 import hashlib
 import itertools
 import math
-from collections import Counter, defaultdict
-from functools import partial
-from typing import (
-    Any,
-    Callable,
-    Collection,
-    Dict,
-    FrozenSet,
-    Generic,
-    Iterable,
-    List,
-    Literal,
-    MutableMapping,
-    Optional,
-    Sequence,
-    Set,
-    Tuple,
-    Type,
-    TypeVar,
-    Union,
-    cast,
-    overload,
-)
+from collections import Counter
+from typing import Callable, Dict, Iterable, List, Optional, Sequence, Set, Tuple, TypeVar, Union, cast, overload
 
 import torch
 from typing_extensions import Concatenate, Never, ParamSpec
@@ -36,16 +15,11 @@ from rust_circuit.py_utils import assert_never, check_cast
 
 from ._rust import (
     Add,
-    Array,
     Circuit,
     Concat,
     ConstructError,
-    Cumulant,
-    DiscreteVar,
     Einsum,
-    GeneralFunction,
     Index,
-    IterativeMatcher,
     IterativeMatcherIn,
     Matcher,
     MatcherIn,
@@ -53,8 +27,6 @@ from ._rust import (
     Rearrange,
     RearrangeSpec,
     Scalar,
-    StoredCumulantVar,
-    Tag,
     TorchAxisIndex,
     add_elim_zeros,
     add_flatten_once,
