@@ -245,7 +245,12 @@ class ExactSampler(CondSampler):
 
 @attrs.define(hash=True, eq=True)
 class FixedOtherSampler(UncondSampler):
+    other: Dataset = attrs.field(
+        kw_only=True
+    )  # irrelevant since we are writing our own init method, but attrs complains otherwise
+
     def __init__(self, other: Dataset):
+        super().__init__()
         self.other = other
 
     def __call__(self, ref: Dataset, ds: Dataset, rng=None) -> Dataset:
