@@ -1,9 +1,8 @@
 import functools
 import json
 import os
-import time
 from dataclasses import dataclass
-from typing import Any, Callable, List, Optional, Protocol, Tuple, TypeVar, Union, cast
+from typing import Any, Callable, List, Optional, Tuple, Union, cast
 
 import torch
 
@@ -284,7 +283,7 @@ class VeryNamedTensor:
 
     def __repr__(self):
         indent = " " * 4  # len("VeryNamedTensor(")
-        str = f"VeryNamedTensor(\n"
+        str = "VeryNamedTensor(\n"
         str += f"{indent}shape={self.shape}\n"
         str += f"{indent}dim_names={self.dim_names}\n"
         str += f"{indent}dim_types={self.dim_types}\n"
@@ -435,7 +434,7 @@ def vnt_guessing_shit(tensor, potential_dims, title="title", units="units", perm
     for dim_length in tensor.shape:
         matching_potentials = [x for x in potential_dims if len(x["idx_names"]) == dim_length]
         if len(matching_potentials) == 0:
-            assert permissive == True, "no matching length dims"
+            assert permissive is True, "no matching length dims"
             dim_idx_names.append([str(i) for i in range(dim_length)])
             dim_types.append("unknown")
             name = "unknown"
