@@ -1,8 +1,5 @@
-import re
-import time
-from typing import Callable, List, Optional
+from typing import Callable, Optional
 
-import numpy as np
 import pytest
 import torch
 
@@ -258,8 +255,7 @@ def test_corr_injective_separate_paths():
     dataset = IntDataset.of_shape((1000, 2), lambda x: 2 * x[0] + 3 * x[1])
 
     ex = Experiment(C, dataset, corr)
-    scrubbed = ex.scrub(1)
-    ex.nodes.check_injective_on_treeified(scrubbed.circuit)
+    ex.scrub(1, treeify=True, check=True)
 
 
 def test_corr_tree_structure_respects_paths():
@@ -549,7 +545,7 @@ def test_spread_over_two_nodes():
     corr.add(C_t, to_C)
     corr.add(A_t, to_A)
 
-    run_experiment(C, dataset, corr, lambda l: l > 0.1, num_examples=50)
+    run_experiment(C, dataset, corr, lambda l: l > 0.1, num_examples=num_examples)
 
 
 def test_large_deviation():
