@@ -1,6 +1,6 @@
 from datetime import timedelta
 from functools import partial
-from typing import Any, Callable, List, Optional, Protocol, Sequence, Set, Tuple, Union, cast
+from typing import Any, Callable, List, Sequence, Set, Tuple, cast
 
 import attrs
 import hypothesis
@@ -8,13 +8,22 @@ import hypothesis.extra.numpy as st_np
 import hypothesis.strategies as st
 import torch
 from hypothesis import example, given
-from torch import tensor
 
-from interp.circuit.algebric_rewrite import CircuitT, CircuitT2
+import interp.circuit.testing.strategies as st_c
+from interp.circuit.circuit import Circuit, CircuitTypeError, MemoizedFn
+from interp.circuit.circuit_compiler.util import FrozenDict, TorchAxisIndex
+from interp.circuit.circuit_utils import evaluate_fn
+from interp.circuit.computational_node import Add, Concat, Einsum, Index, UnaryRearrange, dedup_with_order
+from interp.circuit.constant import ArrayConstant, FloatConstant, One, Zero
+from interp.circuit.sample_transform import RunDiscreteVarAllSpec, SampleTransform
+from interp.circuit.testing.topdown_circuit import CircuitProbs as CP
+from interp.circuit.testing.topdown_circuit import st_Circuit
+from interp.circuit.testing.utils import mark_not_interesting_if, target_diff
+from interp.circuit.var import AutoTag
 from rust_circuit import Add as rAdd
 from rust_circuit import Circuit as rCircuit
 from rust_circuit import Einsum as rEinsum
-from rust_circuit import IterativeMatcher, Matcher, Parser
+from rust_circuit import Matcher, Parser
 from rust_circuit import Scalar as rScalar
 from rust_circuit import apply_in_traversal, new_traversal, replace_outside_traversal_symbols, simp
 from rust_circuit.algebric_rewrite import (
@@ -57,37 +66,11 @@ from rust_circuit.algebric_rewrite import (
     split_to_concat_for_batch,
     zero_empty_add,
 )
+from rust_circuit.interop_rust import py_to_rust, rust_to_py
 
 from .test_rust_rewrite import deterministic_rearrange_noneing
 
 P = Parser()
-import interp.circuit.testing.strategies as st_c
-from interp.circuit.circuit import Circuit, CircuitTypeError, MemoizedFn
-from interp.circuit.circuit_compiler.util import FrozenDict, TorchAxisIndex
-from interp.circuit.circuit_utils import evaluate_fn
-from interp.circuit.computational_node import (
-    Add,
-    Concat,
-    Einsum,
-    GeneralFunction,
-    Index,
-    UnaryRearrange,
-    dedup_with_order,
-    sigmoid_fn,
-)
-from interp.circuit.constant import ArrayConstant, FloatConstant, One, Zero
-from interp.circuit.cumulant import Cumulant
-from interp.circuit.get_update_node import AnyMatcher, EqMatcher
-from interp.circuit.get_update_node import NameMatcher as NM
-from interp.circuit.get_update_node import TypeMatcher
-from interp.circuit.partitioning import partition
-from interp.circuit.sample_transform import RunDiscreteVarAllSpec, SampleTransform
-from interp.circuit.testing.topdown_circuit import CircuitProbs as CP
-from interp.circuit.testing.topdown_circuit import st_Circuit
-from interp.circuit.testing.utils import mark_not_interesting_if, target_diff
-from interp.circuit.var import AutoTag
-from interp.tools.type_utils import assert_never
-from rust_circuit.interop_rust import py_to_rust, rust_to_py
 
 
 def is_zero(c: Circuit):
@@ -1104,6 +1087,7 @@ def test_extract_add(extract_add_args: ExtractAddArgs):
     check_transform_equality(lambda x: extract_add(x, py_to_rust(sub).cast_add()), circuit)
 
 
+@hypothesis.settings(deadline=None)
 @given(extract_add_args=st_extract_add_args())
 def test_extract_add_by_match(extract_add_args: ExtractAddArgs):
     circuit, sub = extract_add_args
@@ -1155,7 +1139,7 @@ def test_nested_einsum_axes_permute(nested_einsum_axes_permute_args: NestedEinsu
 def test_nested_einsum_permute_dups_to_eq_manual():
     x = AutoTag(Zero((2, 3)))
     y = AutoTag(Zero((4, 5, 3)))
-    z = AutoTag(Zero((6, 4, 5)))
+    AutoTag(Zero((6, 4, 5)))
 
     x_y = Einsum.from_axes_tuples((x, (0, 1)), (y, (2, 3, 1)), out_axes=(2, 1, 3, 0))
     x_y_other = Einsum.from_axes_tuples((x, (0, 1)), (y, (2, 3, 1)), out_axes=(0, 1, 3, 2))
