@@ -1,14 +1,22 @@
 import itertools
-import math
 from functools import partial
 from typing import *
 
+import hypothesis
+import hypothesis.strategies as st
 import pytest
 import torch
+from hypothesis import note
+from torch.testing import assert_close
 
-import rust_circuit.optional as op
-from interp.model.gpt_modules import MlpActType, PosEncType
-from interp.tools.data_loading import get_val_seqs
+import interp.circuit.testing.strategies as st_c
+import rust_circuit as rc
+import rust_circuit.module_library as mod_l
+from interp.circuit.print_circuit import PrintCircuit
+from interp.circuit.testing.topdown_circuit import CircuitProbs as CP
+from interp.circuit.testing.utils import mark_not_interesting_if, rand_matcher
+from interp.tools.indexer import I
+from interp.tools.rrfs import RRFS_DIR
 from rust_circuit import (
     Add,
     Array,
@@ -35,17 +43,14 @@ from rust_circuit import (
     ModulePusher,
     ModuleSpec,
     Parser,
-    PrintOptions,
     Rearrange,
     RearrangeSpec,
     RunDiscreteVarAllSpec,
     Sampler,
     Scalar,
     Shape,
-    SimpFnSubset,
     Symbol,
     TorchDeviceDtypeOp,
-    Updater,
     all_children,
     any_children_with_symbolic_sizes,
     broadcast_shapes,
@@ -53,9 +58,7 @@ from rust_circuit import (
     conform_all_modules,
     count_nodes,
     deep_map_preorder,
-    deep_normalize,
     default_nested_module_namer,
-    diff_circuits,
     elim_no_input_module,
     expand_node,
     extract_rewrite,
@@ -66,35 +69,19 @@ from rust_circuit import (
     module_new_bind,
     module_remove_unused_inputs,
     new_traversal,
-    simp,
+    replace_expand_bottom_up_dict,
     strip_names_and_tags,
     substitute_all_modules,
     symbolic_sizes,
 )
-
-P = Parser()
-import hypothesis
-import hypothesis.extra.numpy as st_np
-import hypothesis.strategies as st
-from hypothesis import note
-from torch.testing import assert_close
-
-import interp.circuit.testing.strategies as st_c
-import rust_circuit as rc
-import rust_circuit.module_library as mod_l
-from interp.circuit.print_circuit import PrintCircuit
-from interp.circuit.testing.topdown_circuit import CircuitProbs as CP
-from interp.circuit.testing.topdown_circuit import st_Circuit
-from interp.circuit.testing.utils import mark_not_interesting_if, rand_matcher
-from interp.tools.indexer import I
-from interp.tools.rrfs import RRFS_DIR
-from rust_circuit import replace_expand_bottom_up, replace_expand_bottom_up_dict
 from rust_circuit.interop_rust import rust_to_py
 from rust_circuit.module_library import get_attention, get_bilinear_mlp, get_pointwise_mlp
 from rust_circuit.py_utils import timed
 
 from .test_rust_rewrite import get_c_st
 
+P = Parser()
+
 
 @hypothesis.given(get_c_st(rust=True))
 def test_subst_all_modules(circ):
@@ -479,7 +466,7 @@ def test_expand_node():
         assert out.shape == (13, 12, 9, 10, 8, 9, 7)
         assert out.cast_module().substitute().shape == (13, 12, 9, 10, 8, 9, 7)
 
-    s = f"""
+    s = """
     'rep_fst' [    12,      102, 7] Symbol 0bac3288-00e7-41bf-bd59-ac04de67ba47
     'rep_snd' [13, 12,  10, 102, 7] Symbol d5f29a2c-5927-4237-a7ea-7fe7ec2c3be6
     'a' Module
@@ -679,7 +666,7 @@ def test_extract_gpt():
             ),
             fancy_validate=True,
         )
-        cur_2 = Matcher(f"a{i}").update(
+        Matcher(f"a{i}").update(
             cur,
             lambda x: extract_rewrite(
                 x,
