@@ -7,40 +7,26 @@ from datetime import timedelta
 from typing import Any, Dict, List, Optional, Sequence, Tuple, Union, cast
 from uuid import UUID, uuid4
 
-import attrs
 import hypothesis
 import hypothesis.extra.numpy as st_np
 import hypothesis.strategies as st
 import pytest
 import torch
 from hypothesis import note
-from torch import device, randn, tensor
+from torch import tensor
 from torch.testing import assert_close
 
 import rust_circuit as rc
 import rust_circuit.optional as op
 from interp.circuit.circuit import Circuit, MemoizedFn, Shape
-from interp.circuit.circuit_compiler.util import FrozenDict
 from interp.circuit.circuit_compiler.util import RearrangeSpec as pRearrangeSpec
 from interp.circuit.circuit_utils import cast_circuit, evaluate_fn
-from interp.circuit.computational_node import (
-    Add,
-    Concat,
-    Einsum,
-    GeneralFunction,
-    Index,
-    UnaryRearrange,
-    log_softmax_fn,
-    sigmoid_fn,
-    softmax_fn,
-)
+from interp.circuit.computational_node import Concat, Einsum, GeneralFunction, Index, UnaryRearrange, sigmoid_fn
 from interp.circuit.constant import ArrayConstant, FloatConstant, One, Zero
-from interp.circuit.print_circuit import PrintCircuit, lambda_notation_circuit
 from interp.circuit.testing import strategies as st_c
 from interp.circuit.testing.topdown_circuit import CircuitProbs as CP
 from interp.circuit.testing.topdown_circuit import st_Circuit
 from interp.circuit.testing.utils import mark_not_interesting_if
-from interp.circuit.var import AutoTag
 from interp.tools.indexer import I
 from rust_circuit import Add as rAdd
 from rust_circuit import Array as rArray
@@ -49,17 +35,12 @@ from rust_circuit import Concat as rConcat
 from rust_circuit import ConstructArrayHasReservedSymbolicShapeError, DistributeNoopError
 from rust_circuit import Einsum as rEinsum
 from rust_circuit import GeneralFunction as rGeneralFunction
-from rust_circuit import GeneralFunctionSpec as rGeneralFunctionSpec
 from rust_circuit import Index as rIndex
 from rust_circuit import (
-    Module,
-    ModuleArgSpec,
-    ModuleSpec,
     OptimizationSettings,
     Parser,
     PrintOptions,
     PushDownIndexEinsumNoopError,
-    PushDownIndexError,
     PushDownIndexNoopOnConcatError,
     PushDownIndexNoopOnGeneralFunctionError,
     PushDownIndexRearrangeNotPossibleError,
@@ -74,7 +55,6 @@ from rust_circuit import Scatter as rScatter
 from rust_circuit import SchedulingOOMError
 from rust_circuit import Symbol as rSymbol
 from rust_circuit import (
-    TorchDeviceDtype,
     TorchDeviceDtypeOp,
     add_collapse_scalar_inputs,
     add_deduplicate,
@@ -109,7 +89,6 @@ from rust_circuit import (
     deep_canonicalize,
     deep_heuristic_nest_adds,
     deep_map,
-    deep_maybe_distribute,
     deep_pull_concat,
     deep_pull_concat_messy,
     deep_push_down_index_raw,
@@ -308,9 +287,7 @@ def test_optimize_batch(args):
     print(circ.dtype, circ1.dtype)
     circ.print()
     circ1.print()
-    assert_close(
-        circ.evaluate().to(dtype=torch.float64), optimize_and_evaluate(circ1, FLOAT64_SETTING).to(dtype=torch.float64)
-    )
+    assert_close(circ.evaluate().to(dtype=torch.float64), optimize_and_evaluate(circ1).to(dtype=torch.float64))
 
 
 def raw_test_compily_py(pycirc):
@@ -1948,7 +1925,7 @@ def test_fancy_einsum_string():
 
 def test_mixed_dtype_error():
     with pytest.raises(ValueError):
-        circ = rAdd(
+        rAdd(
             rArray(torch.tensor([1, 2, 3], dtype=torch.float32)),
             rArray(torch.tensor([4, 5, 6], dtype=torch.float64)),
         )
