@@ -15,8 +15,9 @@ use crate::error::ExceptionWithRustContext;
 /// import the module.
 #[macro_use]
 pub mod error;
+
 #[pyo3::pymodule]
-fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
+fn _rust(py: Python<'_>, m: &PyModule) -> PyResult<()> {
     pyo3::anyhow::set_anyhow_to_py_err(Box::new(error::anyhow_to_py_err));
 
     use circuit_base::{
